import 'package:osam_flutter/osam_flutter.dart';
import 'package:osam_flutter/src/navigation/osam_route.dart';

mixin NavigationPresenter<US, AS> on Presenter<US, AS> {
  Navigation get navigation;

  void pop() => navigation.pop();

  void popUntil<T extends OsamRoute>() => navigation.popUntil<T>();

  void setRouteResult<T extends OsamRoute>(Object result) =>
      findRoute<T>().result = result;

  Future<R> push<R>(OsamRoute<R> route) => navigation.push(route);

  Future<R> pushReplacement<R>(OsamRoute<R> route) =>
      navigation.pushReplacement(route);

  Future<R> pushAndRemoveUntil<R>(OsamRoute<R> route) =>
      navigation.pushAndRemoveUntil(route);

  List<OsamRoute<Object>> get currentRoutes => navigation.routes.value;

  OsamRoute<Object> get currentRoute => currentRoutes.last;

  T findRoute<T extends OsamRoute>() =>
      isInStack<T>() ? navigation.findRoute<T>() : null;

  OsamRoute<Object> get firstRoute => currentRoutes.first;

  bool isInStack<T extends OsamRoute>() =>
      currentRoutes.firstWhere((element) => element is T, orElse: () => null) !=
      null;

  bool get canPop => navigation.routes.value.length > 1;

  OsamRoute<Object> get routeBefore => currentRoutes.length > 1
      ? currentRoutes[currentRoutes.length - 2]
      : throw null;
}

mixin TabNavigationPresenter<US, AS> on Presenter<US, AS> {
  TabNavigation get tabNavigation;

  String get currentTab => tabNavigation.tabNotifier.value;

  void changeTab(String newTab) => tabNavigation.tab = newTab;
}
