import 'package:flutter/material.dart';
import 'package:flutter/scheduler.dart';

final _gc = GlobalKey<OverlayState>();
const animationThrottle = Duration(milliseconds: 100);

void showNotification(
    {Widget child,
    EdgeInsetsGeometry padding = const EdgeInsets.all(8),
    Curve curve = Curves.easeIn,
    Duration animationDuration = const Duration(milliseconds: 300),
    Duration stayDuration = const Duration(seconds: 1),
    Duration autoHideDuration = const Duration(seconds: 1),
    EdgeInsets safeAreaPadding}) {
  final entry = OverlayEntry(
      builder: (ctx) => _Notification(
            child: child,
            padding: padding,
            animationDuration: animationDuration,
            stayDuration: stayDuration,
          ));
  _gc.currentState.insert(entry);
  Future.delayed((autoHideDuration + (animationDuration * 2)) +
          animationThrottle +
          stayDuration)
      .then((value) {
    entry.remove();
  });
}

class _Notification extends StatefulWidget {
  final Widget child;
  final EdgeInsetsGeometry padding;
  final Duration animationDuration;
  final Duration stayDuration;

  const _Notification(
      {Key key,
      this.child,
      this.padding,
      this.animationDuration,
      this.stayDuration})
      : super(key: key);

  @override
  __NotificationState createState() => __NotificationState();
}

class __NotificationState extends State<_Notification> {
  var childHeight;
  var isRendered = false;
  var show = false;

  @override
  Widget build(BuildContext context) {
    return Stack(
      fit: StackFit.expand,
      children: [
        isRendered
            ? AnimatedPositioned(
                top: show ? 0 : -childHeight,
                duration: widget.animationDuration,
                onEnd: () {
                  if (show) {
                    Future.delayed(widget.stayDuration, () {
                      setState(() {
                        show = false;
                      });
                    });
                  }
                },
                child: Padding(
                  padding: widget.padding,
                  child: widget.child,
                ),
              )
            : Positioned(
                top: -2000,
                child: MeasureSize(
                  onChange: (size) {
                    childHeight = size.height;
                    setState(() {
                      isRendered = true;
                    });
                    Future.delayed(
                        animationThrottle, () => setState(() => show = true));
                  },
                  child: Padding(
                    padding: widget.padding,
                    child: widget.child,
                  ),
                ),
              )
      ],
    );
  }
}

class OsamNotification extends StatelessWidget {
  final Widget child;

  const OsamNotification({Key key, this.child}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return Directionality(
      textDirection: TextDirection.ltr,
      child: Overlay(
        key: _gc,
        initialEntries: [OverlayEntry(builder: (_) => child)],
      ),
    );
  }
}

typedef void OnWidgetSizeChange(Size size);

class MeasureSize extends StatefulWidget {
  final Widget child;
  final OnWidgetSizeChange onChange;

  const MeasureSize({
    Key key,
    @required this.onChange,
    @required this.child,
  }) : super(key: key);

  @override
  _MeasureSizeState createState() => _MeasureSizeState();
}

class _MeasureSizeState extends State<MeasureSize> {
  @override
  Widget build(BuildContext context) {
    SchedulerBinding.instance.addPostFrameCallback(postFrameCallback);
    return Container(
      key: widgetKey,
      child: widget.child,
    );
  }

  final widgetKey = GlobalKey();
  var oldSize;

  void postFrameCallback(_) {
    final context = widgetKey.currentContext;
    if (context == null) return;

    final newSize = context.size;
    if (oldSize == newSize) return;

    oldSize = newSize;
    widget.onChange(newSize);
  }
}
